package me.unurled.sacredrealms.cutscenes;

import java.util.ArrayList;
import java.util.List;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.format.TextColor;
import org.bukkit.Bukkit;
import org.bukkit.command.Command;
import org.bukkit.command.CommandSender;
import org.bukkit.command.TabExecutor;
import org.bukkit.entity.Player;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

public class CutsceneCommand implements TabExecutor {

    List<String> complete = new ArrayList<>();

    @Override
    public boolean onCommand(@NotNull CommandSender sender, @NotNull Command command,
        @NotNull String label, @NotNull String[] args) {
        if (args.length == 1) {
            // stop, list
            if (args[0].equalsIgnoreCase("stop")) {
                // stop cutscene
                if (sender.hasPermission("cutscenes.use")) {
                    // stop current cutscene current player
                    if (sender instanceof Player p) {
                        if (Cutscenes.getInstance().getCutscene(p) != null) {
                            Cutscenes.getInstance().stopCutscene(p);
                            return true;
                        }
                        return true;
                    }
                }
            } else if (sender.hasPermission("cutscenes.admin")) {
                if (args[0].equalsIgnoreCase("list")) {
                    // list all cutscenes
                    sender.sendMessage(Component.text("Cutscenes: ").append(Component.text(Cutscenes.getInstance().getCutscenesId().toString())));
                } else if (args[0].equalsIgnoreCase("reload")) {
                    Cutscenes.getInstance().stopCutscenesForAllPlayers();
                    Cutscenes.getInstance().load();
                }
                return true;
            }
        } else if (args.length == 2) {
            // play, stop, create, remove
            if (sender.hasPermission("cutscenes.admin")) {
                if (args[0].equalsIgnoreCase("play")) {
                    if (sender instanceof Player p) {
                        Cutscenes.getInstance().playCutscene(
                            Cutscenes.getInstance()
                                .getCutscene(args[1].toUpperCase()),
                            p
                        );
                        return true;
                    }
                } else if (args[0].equalsIgnoreCase("stop")) {
                    if (Bukkit.getOnlinePlayers().contains(Bukkit.getPlayer(args[1]))) {
                        Cutscenes.getInstance().stopCutscene(Bukkit.getPlayer(args[1]));
                        return true;
                    }
                }
            }
        } else if (args.length == 3) {
            if (sender.hasPermission("cutscenes.admin")) {
                if (args[0].equalsIgnoreCase("play")) {
                    if (Bukkit.getOnlinePlayers().contains(Bukkit.getPlayer(args[2]))) {
                        Cutscenes.getInstance().playCutscene(
                            Cutscenes.getInstance()
                                .getCutscene(args[1].toUpperCase()),
                            Bukkit.getPlayer(args[2])
                        );
                        return true;
                    }
                }
            }
        } else {
            // usage/help message
            if (sender.hasPermission("cutscenes.admin")) {
                sender.sendMessage(Component
                    .text("Usage: /cutscenes <play|stop|reload|list> <cutscene_id|player (only play and stop)> <player (only possible if play with an id before)>")
                    .color(TextColor.color(255, 251, 0)));
            } else {
                sender.sendMessage(Component
                    .text("Usage: /cutscenes stop")
                    .color(TextColor.color(255, 251, 0)));
            }
        }
        return true;
    }

    @Override
    public @Nullable List<String> onTabComplete(@NotNull CommandSender sender,
        @NotNull Command command, @NotNull String label, @NotNull String[] args) {
        complete.clear();
        complete.add("stop");
        if (sender.hasPermission("cutscenes.admin")) {
            complete.add("play");
            complete.add("list");
            complete.add("reload");
        }
        List<String> result = new ArrayList<>();
        if (args.length == 1) {
            for (String s : complete) {
                if (s.toLowerCase().startsWith(args[0].toLowerCase())) {
                    result.add(s);
                }
            }
            return result;
        } else if (args.length == 2) {
            if (sender.hasPermission("cutscenes.admin")) {
                if (args[0].equalsIgnoreCase("stop")) {
                    for (Player p : Bukkit.getOnlinePlayers()) {
                        if (p.getName().toLowerCase().startsWith(args[1].toLowerCase())) {
                            result.add(p.getName());
                        }
                    }
                } else if (args[0].equalsIgnoreCase("play")) {
                    for (String s : Cutscenes.getInstance().getCutscenesId()) {
                        if (s.toLowerCase().startsWith(args[1].toLowerCase())) {
                            result.add(s);
                        }
                    }
                }
            }
            return result;
        } else if (args.length == 3) {
            if (sender.hasPermission("cutscenes.admin")) {
                if (args[0].equalsIgnoreCase("play")) {
                    for (Player p : Bukkit.getOnlinePlayers()) {
                        if (p.getName().toLowerCase().startsWith(args[2].toLowerCase())) {
                            result.add(p.getName());
                        }
                    }
                }
            }
        }
        return null;
    }
}
