package me.unurled.sacredrealms.cutscenes.parse;

import me.unurled.sacredrealms.cutscenes.Cutscenes;
import me.unurled.sacredrealms.cutscenes.Frame;
import me.unurled.sacredrealms.cutscenes.Marker;
import org.bukkit.Bukkit;
import org.bukkit.Location;
import org.bukkit.configuration.file.FileConfiguration;

/**
 * ```
 * id: "marker_id"
 * cutscene_id: "cutscene_id"
 * start: "world,x,y,z,yaw,pitch"
 * timeBetweenFrames: "long time"
 * overAllTime: "long time"
 * frames: # in order of play
 *  - "0,world,x,y,z,yaw,pitch"
 *  - "1,world,x,y,z,yaw,pitch"
 *  ...
 * ```
 */

public class MarkerParser {

    public static Marker parse(FileConfiguration file) {
        String id = null;
        Marker marker = null;
        if (file.contains("id")) { // id: "marker_id"
            id = file.getString("id");
        }
        if (id != null && id.length() != 0) {
            try {
                marker = new Marker(Integer.parseInt(id));
            } catch (NumberFormatException e) {
                Cutscenes.getLOGGER().error("Error while parsing marker ID: " + e.getMessage());
                return null;
            }
        } else {
            Cutscenes.getLOGGER().error("Error while parsing marker ID " + (id == null ? "(id)" : id));
            return null;
        }
        double x = 0, y = 0, z = 0;
        float yaw = 0, pitch = 0;
        if (file.contains("start")) { // start: "world,x,y,z,yaw,pitch"
            try {
                String[] start = file.getString("start").split(",");
                x = Double.parseDouble(start[1]);
                y = Double.parseDouble(start[2]);
                z = Double.parseDouble(start[3]);
                yaw = Float.parseFloat(start[4]);
                pitch = Float.parseFloat(start[5]);
                Location location = new Location(Bukkit.getWorld(start[0]), x, y, z, yaw, pitch);
                marker.setStart(location);
            } catch (Exception e) {
                Cutscenes.getLOGGER().error("Error while parsing cutscene start location: " + e.getMessage());
                return null;
            }
        }
        if (file.contains("timeBetweenFrames")) { // timeBetweenFrames: "long time"
            try {
                marker.setTimeBetweenFrames(Long.parseLong(file.getString("timeBetweenFrames", "50")));
            } catch (NumberFormatException e) {
                Cutscenes.getLOGGER().error("Error while parsing cutscene timeBetweenFrames: " + e.getMessage());
                return null;
            }
        }
        if (file.contains("overAllTime")) { // overAllTime: "long time"
            try {
                marker.setOverAllTime(Long.parseLong(file.getString("overAllTime", "250")));
            } catch (NumberFormatException e) {
                Cutscenes.getLOGGER().error("Error while parsing cutscene overAllTime: " + e.getMessage());
                return null;
            }
        }
        return marker;
    }
}
